/*
 * This file is part of SubLink.
 * Copyright (C) 2009  Jeferson Hultmann <hultmann@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

"use strict";

var EXPORTED_SYMBOLS = ["Cc", "Ci", "util", "console", "findWordPosition"];

var Cc = Components.classes;
var Ci = Components.interfaces;

Components.utils.import("resource://gre/modules/Services.jsm");


function findWordPosition(txt, idx) {
  var bounds = [" ", "\n", "\t", "(", ")", '"', "'", "<", ">", "[", "]"];
  var mySort = function(a, b) {
    return a - b;
  };

  var stopChars = bounds.map(function(myChar) {
    var idx2 = txt.indexOf(myChar, idx);
    return idx2 === -1 ? Infinity : idx2;
  });


  stopChars.sort(mySort);
  var endNearest = stopChars[0];
  var lastChar = (endNearest === Infinity ? txt.length : endNearest) - 1;


  var startChars = bounds.map(function(myChar) {
    return txt.lastIndexOf(myChar, lastChar);
  });

  startChars.sort(mySort);
  var startNearest = startChars[startChars.length - 1];

  var firstChar = startNearest == -1 ? 0 : startNearest + 1;
  return [firstChar, lastChar - firstChar + 1];
}



var util = {

  checkSecurity: function (uri) {
    switch (uri.scheme) {
      case "javascript":
      case "data":
        return false;
    }
    return true;
  },


  toClipboard: function(txt) {
    Cc["@mozilla.org/widget/clipboardhelper;1"].getService(Ci.nsIClipboardHelper).copyString(txt);
  },


  isOffline: function() {
    return Services.io.offline;
  },


  openPage: function(url, where, chromeWin) {
    console.log("openPage =", url);
    if (util.checkSecurity(util.stringToUri(url)) === false) {
      console.log("openPage security false ", url);
      return;
    }

    if (chromeWin === null) {
      chromeWin = Services.wm.getMostRecentWindow("navigator:browser");
    }

    //var referer = chromeWin.getBrowser().selectedBrowser.contentDocument.documentURIObject;
    chromeWin.openUILinkIn(url, where);
    chromeWin.content.focus();
  },


  openPage2: function(evt, url, close) {
    var menuItem = evt.target;
    var chromeWin = menuItem.ownerDocument.defaultView;
    var where;

    switch (evt.type) {
      case "click":
        if ((menuItem.getAttribute("disabled") === "true")
        || (evt.button !== 1)) {
          return; // do nothing
        }
        where = "tabshifted";
        break;
      case "command":
        where = "current";
        break;
      default:
        console.trace();
        throw new Error(evt.type);
    }

    if (close) {
      chromeWin.closeMenus(menuItem);
    }

    util.openPage(url, where, chromeWin);
  },


  pageTitle: function(uri, callback) {
    var asyncHistory = Cc["@mozilla.org/browser/history;1"].getService(Ci.mozIAsyncHistory);
    asyncHistory.getPlacesInfo(uri, {
      handleResult: function(placeInfo) {
        callback(placeInfo.title);
      },
      handleError: function(resultCode, placeInfo) {
        callback(null);
      },
      handleCompletion: function() {}
    });
  },


  stringToUri: function(spec) {
    try {
      return Services.io.newURI(spec, null, null);
    } catch (ex) {
      return null;
    }
  },


  getText: function(msg) {
    var qty = arguments.length - 1;
    var args = new Array(qty);
    for (var idx = 0; idx < qty; idx++) {
      args[idx] = arguments[idx + 1];
    }
    return Services.strings
            .createBundle("chrome://sublink-migreme-4.1.0/locale/general.properties")
            .formatStringFromName(msg, args, args.length);
  }

};



var console = {
  log: function() {
    var now = new Date();
    var ms = now.getMilliseconds();
    var ms2;
    if (ms < 100) {
      ms2 = ms < 10 ? "00" + ms : "0" + ms;
    } else {
      ms2 = ms.toString();
    }
    var p = "sublink-migreme[" + now.toLocaleFormat("%H:%M:%S") + "." + ms2 + "] ";

    var len = arguments.length;
    var msg = len > 1 ? Array.prototype.slice.call(arguments, 0, len).join(" ")
                      : arguments[0];
    Services.console.logStringMessage(p + msg);
  },

  warn: function(msg) {
    var message = Cc["@mozilla.org/scripterror;1"].createInstance(Ci.nsIScriptError2);
    message.init(msg,
                 null, // sourceName
                 null, // sourceLine
                 0, 0, // line, col
                 Ci.nsIScriptError.warningFlag,
                 "component javascript");
    Services.console.logStringMessage(message);
    this.trace(msg);
  },

  error: function(msg) {
    var ex = new Error(msg);
    Components.utils.reportError(ex);
    this.trace(msg);
  },

  assert: function(test, msg) {
    if (test !== true) {
      var ex =  new Error("console.assert - " + msg + " - " + test);
      Components.utils.reportError(ex); // workaround - sometimes exception doesn't show up in console
      console.trace("console.assert()");
      throw ex;
    }
  },

  trace: function(desc) {
    var b = [];
    for (var s = Components.stack; s; s = s.caller) {
      b.push(s);
    }

    var padding = "";
    var t = new Array(b.length);
    for (var idx = b.length - 1; idx > -1; idx--) {
      var s = b[idx];
      var name = s.name === null ? "(anonymous)" : s.name;
      t[idx] = s.languageName + " " + padding + s.filename + "\t\t" + name + "\t" + s.lineNumber;
      padding += " ";
    }
    if (!desc) {
      desc = "console.trace()";
    }
    console.log(desc + "\n" + t.reverse().join("\n"));
  }
};
